/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <string.h>
#include <signal.h>
#include <stdio.h>

#include <stream.h>
#include <sys/mman.h>

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

#ifndef __cluster_HXX
#include "cluster.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __page_HXX
#include "page.hxx"
#endif

#ifndef __obj_HXX
#include "obj.hxx"
#endif

#ifndef __volume_HXX
#include "volume.hxx"
#endif

#ifndef __dbserver_HXX
#include "dbserver.hxx"
#endif

#ifndef __volallocator_HXX
#include "volallocator.hxx"
#endif

#ifndef __yoo_root_HXX
#include "yoo_root.hxx"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#include "dictionary.hxx"

extern void genVol() ;
extern void delvol(char*, int recover) ;


void create(char* base, int recover) {

  delvol(base, recover) ;
  
  FILE* f_vol = fopen(YOO_VOL_FILE, "w") ;
  if (f_vol == NULL) {
    perror ("Error : Cannot open volume file") ;
    exit(1) ;
  }
  
  fprintf(f_vol, "server yooda {\n") ;
  fprintf(f_vol, "\tlog %s ;\n", YOO_SYSTEM_DIR) ;
  fprintf(f_vol, "\tvolume admin {\n") ;
  fprintf(f_vol, "\t\tident 1 ;\n") ;
  fprintf(f_vol, "\t\tfile %s/%s_admin ;\n",YOO_SYSTEM_DIR, YOODA_BASE) ;
  fprintf(f_vol, "\t}\n") ;
  fprintf(f_vol, "\tvolume defaultVol {\n") ;
  fprintf(f_vol, "\t\tident 2 ;\n") ;
  fprintf(f_vol, "\t\tfile %s/%s_defaultVol ;\n",YOO_VOL_DIR, YOODA_BASE) ;
  fprintf(f_vol, "\t}\n") ;
  fprintf(f_vol, "}\n") ;

  fclose(f_vol) ;
}


void createCluster() {

  Cluster clu(YOO_ADMIN_VOL) ;  
  clu.getOid() = OID(YOO_ADMIN_VOL, YOO_FIRST_USED_PAGE, 0) ;
  int size = yoo_slotSize(sizeof(Cluster)) ;
  Page p ;
  PID firstpid(YOO_ADMIN_VOL, YOO_FIRST_USED_PAGE) ;
  p.init(&clu, firstpid, 1) ;
  ObjectX* obj = p.allocObject(size) ;
  obj->setClass(UNTYPED);
  ((Cluster*)obj)->setVol(YOO_ADMIN_VOL) ;
  Volume* vol = YOO_DBServer->volume(YOO_ADMIN_VOL) ;
  PID pid = vol->alloc(1) ;
  if (pid != firstpid) {
    yoo_error(YOO_ERR_DBG, "Error during init") ;
    exit(YOO_ERR_DBG) ;
  }
  vol->write(&p, pid) ;
  vol->sync() ;
}


void option_error(char* name) 
{
  printf("usage: %s [-v] [-s] -b <basename>\n", name);
  exit(1) ;
}


int main(int argc, char** argv) {

  char* base = NULL ;
  int recover = FALSE ;
  int vers = 0 ;
  
  for (int i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-b") == 0) {
      base = argv[i+1] ;
      i++ ;
    } else if (strcmp(argv[i], "-v") == 0) {
      vers = 1 ;
    } else if (strcmp(argv[i], "-s") == 0) {
      recover = TRUE ;
    } else option_error(argv[0]) ;
  } 

  if (base == NULL) option_error(argv[0]) ;
  
  if (vers) {
    printf("YOODA version 1.2 <%s>, Copyright (C) 1994 Eric Abecassis
YOODA comes with ABSOLUTELY NO WARRANTY; for details type `COPYING'
in YOODA_HOME directory. This is free software, and you are welcome to 
redistribute it under certain conditions; for details read `COPYING'
or 'COPYING.LIB' in YOODA_HOME directory\n", argv[0]) ;
  }

  create(base, recover) ;

  if (recover) {
    cerr << "-> init Base : " << YOODA_BASE << endl ;
    cerr << "<- init done" << endl ;
    yoo_connect(base, MONOUSER) ;
    genVol() ;
    TRY {
      while (YoodaDictionary->dictName.count() != 0) {
	delete (ADM_NameObj*) YoodaDictionary->dictName.get(0) ;
      }
    } EXCEPTION {
      switch (__YOO_ERR) {
      default :
	break ;
      }
    }
    yoo_commit() ;
    yoo_disconnect() ;
    exit(0);
  }
  
  yoo_connect(base, MONOUSER) ;

  cerr << "-> init Base : " << YOODA_BASE << endl ;
  createCluster() ;

  new Cluster(YOO_TMP_VOL) ;
  cout << "create temporary cluster" << endl;
  new Cluster(YOO_DEF_VOL) ;

  char* volname = "defaultVol" ;
  char* cluname = "defaultClu" ;
  char volfile[500] ;

  sprintf(volfile, "%s/%s_%s",YOO_VOL_DIR, base, volname) ;

  YoodaDictionary = new ADM_Admin ;
  YoodaDictionary->defServ = new ADM_Server("yooda") ;
  sprintf(volfile, "%s/%s_%s",YOO_VOL_DIR, base, volname) ;
  YoodaDictionary->defVol = new ADM_Vol(volname, volfile) ;
  YoodaDictionary->defVol->volId = YOO_DEF_VOL ;
  sprintf(volfile, "%s/%s_%s",YOO_SYSTEM_DIR, base, "admin") ;
  ADM_Vol* vol = new ADM_Vol("admin", volfile) ;
  vol->volId = YOO_ADMIN_VOL ;
  YoodaDictionary->defClu=new ADM_Clu(cluname,YoodaDictionary->defVol);

  yoo_commit() ;
  
  yoo_disconnect() ;

  cerr << "<- init done" << endl ;
}
